﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.IO;
using System.Windows;
using System.Windows.Input;

namespace ACO
{
    internal class OpenProblemClass
    {
        #region Methods

        /// <summary>
        /// reads a problem from a file and loads it into the ant colony
        /// </summary>
        /// <param name="filePath"></param>
        internal void ReadProblemFromFile(string filePath)
        {
            if (filePath == "")
            {
                return;
            }
            FileStream f = new FileStream(filePath, FileMode.Open);
            StreamReader sr = new StreamReader(f);
            string inputLine = sr.ReadLine();
            if (inputLine.StartsWith("TSP"))
            {
                AntColony.Instance.ProblemTyp = ProblemType.TSP;
            }
            else
            {
                if (inputLine.StartsWith("VRP"))
                {
                    AntColony.Instance.ProblemTyp = ProblemType.VRP;
                }
                else
                {
                    //unrecognized problem
                    return;
                }
            }
            while ((inputLine = sr.ReadLine()) != null)
            {
                switch ((char)inputLine[0])
                {
                    case '#': //comments
                        //do nothing
                        break;
                    case '!': //parameter
                        if (inputLine.StartsWith("!mrav"))
                        {
                            AntColony.Instance.AntNumber=int.Parse(inputLine.Split(' ')[1]);
                        }
                        if (inputLine.StartsWith("!alpha"))
                        {
                            AntColony.Instance.Alpha = double.Parse(inputLine.Split(' ')[1]);
                        }
                        if (inputLine.StartsWith("!beta"))
                        {
                            AntColony.Instance.Beta = double.Parse(inputLine.Split(' ')[1]);
                        }
                        if (inputLine.StartsWith("!rho"))
                        {
                            AntColony.Instance.Rho = double.Parse(inputLine.Split(' ')[1]);
                        }
                        if (inputLine.StartsWith("!vozila"))
                        {
                            AntColony.Instance.AgentNumber = int.Parse(inputLine.Split(' ')[1]);
                        }
                        break;
                    default: //City
                        double x = double.Parse(inputLine.Split(' ')[0]);
                        double y = double.Parse(inputLine.Split(' ')[1]);
                        AntColony.Instance.cityList.Add(new City(x, y));
                        break;
                }
            }
            sr.Close();
            f.Close();
        }

        /// <summary>
        /// creates a problem with 3 circles within each other, with aproximatley the 
        /// given number of citys and loads it into the colony
        /// </summary>
        /// <param name="cityNumber">target number of citys</param>
        internal void MakeCircle(int cityNumber)
        {
            AntColony.Instance.ProblemTyp = ProblemType.TSP;
            AntColony.Instance.AgentNumber = 3;
            AntColony.Instance.cityList.Clear();
            int j = 0;
            double X_ord = 0;
            double Y_ord = 0;
            int cityInQuadrant = (int)Math.Round(cityNumber / 12.0);
            for (j = 0; j < cityInQuadrant + 1; j++)
            {
                X_ord = 220 * (Math.Cos(j * 3.1412 / (cityInQuadrant * 2)));
                Y_ord = 220 * (Math.Sin(j * 3.1412 / (cityInQuadrant * 2)));
                AntColony.Instance.cityList.Add(new City(275 + X_ord, 275 + Y_ord));
                AntColony.Instance.cityList.Add(new City(275 + X_ord / 3, 275 + Y_ord / 3));
                AntColony.Instance.cityList.Add(new City(275 + X_ord * 2 / 3, 275 + Y_ord * 2 / 3));
                if (j < cityInQuadrant)
                {
                    AntColony.Instance.cityList.Add(new City(275 - X_ord, 275 + Y_ord));
                    AntColony.Instance.cityList.Add(new City(275 - X_ord / 3, 275 + Y_ord / 3));
                    AntColony.Instance.cityList.Add(new City(275 - X_ord * 2 / 3, 275 + Y_ord * 2 / 3));
                }
                if (j > 0)
                {
                    if (j < cityInQuadrant)
                    {
                        AntColony.Instance.cityList.Add(new City(275 + X_ord, 275 - Y_ord));
                        AntColony.Instance.cityList.Add(new City(275 + X_ord / 3.0, 275 - Y_ord / 3.0));
                        AntColony.Instance.cityList.Add(new City(275 + X_ord * 2 / 3.0, 275 - Y_ord * 2 / 3.0));
                    }
                    AntColony.Instance.cityList.Add(new City(275 - X_ord, 275 - Y_ord));
                    AntColony.Instance.cityList.Add(new City(275 - X_ord / 3.0, 275 - Y_ord / 3.0));
                    AntColony.Instance.cityList.Add(new City(275 - X_ord * 2 / 3.0, 275 - Y_ord * 2 / 3.0));
                }
            }
            return;
        }

        /// <summary>
        /// creates a problem with mesh ordered citys, with aproximatley the 
        /// given number of citys and loads it into the colony
        /// </summary>
        /// <param name="citynumber">target number of citys</param>
        internal void MakeCube(int citynumber)
        {
            AntColony.Instance.ProblemTyp = ProblemType.TSP;
            int i = 0;
            int j = 0;
            int gridNumber = (int)Math.Round(Math.Sqrt(citynumber));
            double space = 0.89 / gridNumber;
            for (i = 0; i < gridNumber; i++)
            {
                for (j = 0; j < gridNumber; j++)
                {
                    AntColony.Instance.cityList.Add(new City(space / 10 + i * space, space / 10 + j * space));
                }
            }
            return;
        }

        /// <summary>
        /// creates the given number of random citys and loads the problem into the colony
        /// </summary>
        /// <param name="cityNumber">number of citys</param>
        internal void MakeRandom(int cityNumber)
        {
            int i = 0;
            double x = 0;
            double y = 0;
            for (i = 0; i < cityNumber; i++)
            {
                x = AntColony.Instance.randomizer.NextDouble();
                y = AntColony.Instance.randomizer.NextDouble();
                AntColony.Instance.cityList.Add(new City(x, y));
            }
            if (AntColony.Instance.randomizer.Next(2) == 0)
            {
                AntColony.Instance.ProblemTyp = ProblemType.TSP;
            }
            else
            {
                AntColony.Instance.ProblemTyp = ProblemType.VRP;
            }
        }

        /// <summary>
        /// creates the ACO-letter-problem and loads it into the colony
        /// </summary>
        internal void MakeACOLetters()
        {
            // A
            AntColony.Instance.cityList.Add(new City(0.150, 0.150));
            AntColony.Instance.cityList.Add(new City(0.110, 0.350));
            AntColony.Instance.cityList.Add(new City(0.190, 0.350));
            AntColony.Instance.cityList.Add(new City(0.120, 0.300));
            AntColony.Instance.cityList.Add(new City(0.180, 0.300));
            AntColony.Instance.cityList.Add(new City(0.170, 0.250));
            AntColony.Instance.cityList.Add(new City(0.160, 0.200));
            AntColony.Instance.cityList.Add(new City(0.130, 0.250));
            AntColony.Instance.cityList.Add(new City(0.140, 0.200));
            AntColony.Instance.cityList.Add(new City(0.142, 0.250));
            AntColony.Instance.cityList.Add(new City(0.158, 0.250));
            // C
            AntColony.Instance.cityList.Add(new City(0.240, 0.190));
            AntColony.Instance.cityList.Add(new City(0.230, 0.250));
            AntColony.Instance.cityList.Add(new City(0.240, 0.310));
            AntColony.Instance.cityList.Add(new City(0.260, 0.350));
            AntColony.Instance.cityList.Add(new City(0.260, 0.150));
            AntColony.Instance.cityList.Add(new City(0.300, 0.350));
            AntColony.Instance.cityList.Add(new City(0.300, 0.150));
            AntColony.Instance.cityList.Add(new City(0.340, 0.320));
            AntColony.Instance.cityList.Add(new City(0.340, 0.180));
            // O
            AntColony.Instance.cityList.Add(new City(0.410, 0.190));
            AntColony.Instance.cityList.Add(new City(0.400, 0.250));
            AntColony.Instance.cityList.Add(new City(0.410, 0.310));
            AntColony.Instance.cityList.Add(new City(0.440, 0.350));
            AntColony.Instance.cityList.Add(new City(0.440, 0.150));
            AntColony.Instance.cityList.Add(new City(0.480, 0.350));
            AntColony.Instance.cityList.Add(new City(0.480, 0.150));
            AntColony.Instance.cityList.Add(new City(0.510, 0.310));
            AntColony.Instance.cityList.Add(new City(0.520, 0.250));
            AntColony.Instance.cityList.Add(new City(0.510, 0.190));
            return;
        }

        #endregion

        #region Singleton statics

        private static OpenProblemClass instance;

        /// <summary>
        /// Singleton design pattern, becoure there is never a need for more then one opening class object
        /// </summary>
        public static OpenProblemClass Instance
        {
            get
            {
                if (instance == null)
                {
                    instance = new OpenProblemClass();
                }
                return instance;
            }
        }

        #endregion
    }
}
