﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows;
using System.Windows.Input;

namespace ACO
{
    public class SaveDataClass
    {
        #region fields and attributes

        private static SaveDataClass instance;

        public string saveString;

        #endregion

        #region methods

        /// <summary>
        /// adds the current results from the colony into the saveString in a readable format, and 
        /// calls the Save function with the given filePath
        /// </summary>
        /// <param name="filePath">file to save the data to</param>
        /// <returns>true if succesfull save, false otherwise</returns>
        public Boolean SaveReadableResults(string filePath)
        {
            ResultFound best = AntColony.Instance.BestRouteList[AntColony.Instance.BestRouteList.Count - 1];
            saveString += "najbolja ruta: " + best.routeLength.ToString() + "\r\n";
            saveString += "nadjena u ciklusu: " + best.CyclusesNeeded.ToString() + "\r\n";
            saveString += "nadjena u trenutku: " + best.timeNeeded.ToString() + "\r\n";
            saveString += "broj gradova: " + AntColony.Instance.cityList.Count.ToString() + "\r\n";
            saveString += "broj vozila: " + AntColony.Instance.AgentNumber.ToString() + "\r\n";
            saveString += "broj mrava: " + AntColony.Instance.AntNumber.ToString() + "\r\n";
            saveString += "alpha: " + AntColony.Instance.Alpha.ToString() + "\r\n";
            saveString += "beta: " + AntColony.Instance.Beta.ToString() + "\r\n";
            saveString += "rho: " + AntColony.Instance.Rho.ToString() + "\r\n";
            saveString += "broj ciklusa: " + AntColony.Instance.StopCondition.cyclusNumber.ToString() + "\r\n";
            saveString += "konstanta: " + AntColony.Instance.PheromoneFaktor.ToString() + "\r\n";
            saveString += "Tip algoritma: " + AntColony.Instance.AlgType.AsString() + "\r\n";
            saveString += "Tip feromona: " + AntColony.Instance.PheromoneTyp.ToString() + "\r\n" + "\r\n";
            return this.Save(filePath);
        }

        /// <summary>
        /// adds the current results from the colony into the saveString in a table-friendly format, and 
        /// calls the Save function with the given filePath
        /// </summary>
        /// <param name="filePath">file to save the data to</param>
        /// <returns>true if succesfull save, false otherwise</returns>
        public Boolean SaveTableResults(string filePath)
        {
            ResultFound best = AntColony.Instance.BestRouteList[AntColony.Instance.BestRouteList.Count-1];
            if (saveString == "")
            {
                saveString += "najbolja ruta;nadjena u ciklusu;nadjena u trenutku;broj gradova;broj vozila;broj mrava;alpha;beta;rho;broj ciklusa;konstanta feromona;Tip algoritma;Tip feromona" + "\r\n";
            }
            saveString += best.routeLength.ToString() + ";";
            saveString += best.CyclusesNeeded.ToString() + ";";
            saveString += best.timeNeeded.ToString() + ";";
            saveString += AntColony.Instance.cityList.Count.ToString() + ";";
            saveString += AntColony.Instance.AgentNumber.ToString()+";";
            saveString += AntColony.Instance.AntNumber.ToString() + ";";
            saveString += AntColony.Instance.Alpha.ToString() +";";
            saveString += AntColony.Instance.Beta.ToString() + ";";
            saveString += AntColony.Instance.Rho.ToString() + ";";
            saveString += AntColony.Instance.StopCondition.cyclusNumber.ToString() + ";";
            saveString += AntColony.Instance.PheromoneFaktor.ToString() + ";";
            saveString += AntColony.Instance.AlgType.AsString() + ";";
            saveString += AntColony.Instance.PheromoneTyp.ToString() + "\r\n";
            return this.Save(filePath);
        }

        /// <summary>
        /// saves the recorded saveString to the target file, defined by filePath
        /// </summary>
        /// <param name="filePath">file to save the data to</param>
        /// <returns>true if succesfull save, false otherwise</returns>
        public Boolean Save(string filePath)
        {
            try
            {
                FileStream f = new FileStream(filePath, FileMode.OpenOrCreate, FileAccess.Write);
                StreamWriter sw = new StreamWriter(f);
                sw.Write(saveString);
                sw.Close();
                f.Close();
            }
            catch
            {
                return false;
            }
            return true;
        }

        /// <summary>
        /// adds the results from ant colony into the save data class without saving the data,
        /// so it could accumulate and be saved afterwards all together with the "Save(filePath)" function
        /// </summary>
        public void AddForSeries()
        {
            ResultFound best = AntColony.Instance.BestRouteList[AntColony.Instance.BestRouteList.Count - 1];
            if (saveString == "")
            {
                saveString += "najbolja ruta;nadjena u ciklusu;nadjena u trenutku;broj gradova;broj vozila;broj mrava;alpha;beta;rho;broj ciklusa;konstanta feromona;Tip algoritma;Tip feromona" + "\r\n";
            }
            saveString += best.routeLength.ToString() + ";";
            saveString += best.CyclusesNeeded.ToString() + ";";
            saveString += best.timeNeeded.ToString() + ";";
            saveString += AntColony.Instance.cityList.Count.ToString() + ";";
            saveString += AntColony.Instance.AgentNumber.ToString() + ";";
            saveString += AntColony.Instance.AntNumber.ToString() + ";";
            saveString += AntColony.Instance.Alpha.ToString() + ";";
            saveString += AntColony.Instance.Beta.ToString() + ";";
            saveString += AntColony.Instance.Rho.ToString() + ";";
            saveString += AntColony.Instance.StopCondition.cyclusNumber.ToString() + ";";
            saveString += AntColony.Instance.PheromoneFaktor.ToString() + ";";
            saveString += AntColony.Instance.AlgType.AsString() + ";";
            saveString += AntColony.Instance.PheromoneTyp.ToString() + "\r\n";
        }

        /// <summary>
        /// constructor, initializes the save data to a empty string
        /// </summary>
        private SaveDataClass()
        {
            this.saveString = "";
        }

        /// <summary>
        /// resets the stored data of found results
        /// </summary>
        public void ResetSaveString()
        {
            this.saveString = "";
        }

        #endregion

        #region Attributes

        /// <summary>
        /// singleton design pattern to simplify the use of saveData functions from different windows
        /// </summary>
        public static SaveDataClass Instance
        {
            get
            {
                if (instance == null)
                {
                    instance = new SaveDataClass();
                }
                return instance;
            }
        }

        #endregion
    }
}
