﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows;
using System.Windows.Input;

namespace ACO
{
    public class SaveProblemClass
    {
        #region Methods and constructors

        /// <summary>
        /// creates a textual representation of the loaded problem (in the ant colony)
        /// and saves it to the target file (given by filePath)
        /// </summary>
        /// <param name="filePath">file to save the problem to</param>
        /// <returns>true if saving was succesfull, false otherwise</returns>
        public Boolean SaveProblem(string filePath)
        {
            string Output = "";
            if (AntColony.Instance.ProblemTyp == ProblemType.TSP)
            {
                Output += "TSP" + '\r' + '\n';
            }
            if (AntColony.Instance.ProblemTyp == ProblemType.VRP)
            {
                Output += "VRP" + '\r' + '\n';
                Output += "!vozila" + AntColony.Instance.AgentNumber.ToString() + '\r' + '\n';
            }
            foreach (City c in AntColony.Instance.cityList)
            {
                Output += c.X.ToString() + " ";
                Output += c.Y.ToString() + "\r\n";
            }
            #region saving
            try
            {
                FileStream f = new FileStream(filePath, FileMode.OpenOrCreate, FileAccess.Write);
                StreamWriter sw = new StreamWriter(f);
                sw.Write(Output);
                sw.Close();
                f.Close();
            }
            catch
            {
                return false;
            }
            #endregion
            return true;
        }

        /// <summary>
        /// default constructor, empty but private, for Singleton use only
        /// </summary>
        private SaveProblemClass()
        {

        }

        #endregion

        #region Singleton

        /// <summary>
        /// singleton design pattern, becouse there is never a need for two objects for saving the loaded problems
        /// </summary>
        public static SaveProblemClass Instance
        {
            get
            {
                if (instance == null)
                {
                    instance = new SaveProblemClass();
                }
                return instance;
            }
        }

        private static SaveProblemClass instance;

        #endregion
    }
}
