﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace ACO
{
    public class StopConditions
    {
        #region Fields and Attributes

        public Boolean timeSpan;
        public Boolean cyclusCount;
        public Boolean stopFile;
        public TimeSpan optimizationTime;
        public int cyclusNumber;
        public string stopFilePath;

        #endregion

        #region Methods

        /// <summary>
        /// fully loaded constructor
        /// </summary>
        /// <param name="time">sets the time limit stop condition</param>
        /// <param name="cyclus">sets the cyclus number stop condition</param>
        /// <param name="stopMeFile">sets the stop.me file stop condition</param>
        public StopConditions(Boolean time, Boolean cyclus, Boolean stopMeFile)
        {
            this.timeSpan = time;
            this.cyclusCount = cyclus;
            this.stopFile = stopMeFile;
            if ((time || cyclus) == false)
            {
                this.stopFile = true; //insurace to have at least one stop condition
            }
            this.stopFilePath = "stop.me";
        }

        /// <summary>
        /// is being checked by the ant colony in every cycle; the stop 
        /// conditions check themselves if they are met
        /// </summary>
        /// <returns>true if any of the given stop conditions are met, false otherwise</returns>
        internal Boolean IsStopConditionTrue()
        {
            if (this.timeSpan)
            {
                if (DateTime.Now - AntColony.Instance.startedAt > this.optimizationTime)
                {
                    return true;
                }
            }
            if (this.cyclusCount)
            {
                if (AntColony.Instance.cyclusCounter >= this.cyclusNumber)
                {
                    return true;
                }
            }
            if (this.stopFile)
            {
                FileInfo fi = new FileInfo(stopFilePath);
                if (fi.Exists)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            return false;
        }

        #endregion

    }
}
