﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Win32;
using ACO;

namespace WPF_Window
{
    public partial class Window1 : Window
    {
        private double mapXfactor;
        private double mapYfactor;

        public Window1()
        {
            InitializeComponent();
            this.ReadParameters();
            this.UpdateResults();
            this.mapXfactor =1/600.0;
            this.mapYfactor =1/600.0;
        }
        
        private void DrawProblemCanvas_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            Point city = e.GetPosition(this.DrawProblemCanvas);
            AntColony.Instance.AddCity(city.X*mapXfactor, city.Y*mapYfactor);
            AntColony.Instance.Reset();
            this.UpdateResults();
        }

        #region check boxes

        private void DisplayCitysCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            this.DrawMap();
        }

        private void DisplayBestRouteCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            this.DrawMap();
        }

        private void DisplayTrailCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            this.DrawMap();
        }

        private void DisplayGraphBestCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            this.DrawGraph();
        }

        private void DisplayGraphAverageCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            this.DrawGraph();
        }

        private void DisplayGraphOptimumCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            this.DrawGraph();
        }

        #endregion

        #region Navigation Buttons

        private void ChangeProblemType_Click(object sender, RoutedEventArgs e)
        {
            if (AntColony.Instance.ProblemTyp == ProblemType.TSP)
            {
                AntColony.Instance.ProblemTyp = ProblemType.VRP;
            }
            else
            {
                if (AntColony.Instance.ProblemTyp == ProblemType.VRP)
                {
                    AntColony.Instance.ProblemTyp = ProblemType.TSP;
                }
            }
            AntColony.Instance.Reset();
            this.UpdateResults();
        }

        private void HelpButton_Click(object sender, RoutedEventArgs e)
        {
            HelpWindow sw = new HelpWindow();
            sw.Show();
        }

        private void Reset_Click(object sender, RoutedEventArgs e)
        {
            AntColony.Instance.Reset();
            this.UpdateResults();
        }

        private void StartButton_Click(object sender, RoutedEventArgs e)
        {
            StopConditions sc = new StopConditions((bool)this.StopOnTimeCheckBox.IsChecked, (bool)this.StopOnCyclusCheckBox.IsChecked, (bool)this.StopMeFileCheckBox.IsChecked);
            try
            {
                if ((bool)this.StopOnTimeCheckBox.IsChecked)
                {
                    sc.optimizationTime = new TimeSpan(0, 0, int.Parse(this.StopTimeLimitBox.Text));
                }
                if ((bool)this.StopOnCyclusCheckBox.IsChecked)
                {
                    sc.cyclusNumber = int.Parse(this.StopCyclusLimitBox.Text)+AntColony.Instance.CyclusCounter;
                }
            }
            catch
            {
                MessageBox.Show("Error in reading stop condition values!\ncheck for characters and/or floating points in integer values");
            }
            AlgType at;
            if ((bool)this.AlgGreedyRadio.IsChecked == true)
            {
                at = new AlgTypeGreedy();
            }
            else
            {
                at = new AlgTypeACO((bool)this.AlgElitismCheckBox.IsChecked, (bool)this.AlgFootSteppingCheckBox.IsChecked, (bool)this.AlgMinMaxCheckBox.IsChecked, (bool)this.AlgSystemCheckBox.IsChecked);
            }
            PheromoneType pt = new PheromoneType();
            AntColony.Instance.Start(at, pt, sc);
            this.UpdateResults();
        }

        private void StepButton_Click(object sender, RoutedEventArgs e)
        {
            StopConditions sc = new StopConditions(false, true, (bool)this.StopMeFileCheckBox.IsChecked);
            sc.cyclusNumber = AntColony.Instance.CyclusCounter + 1;
            AlgType at;
            if ((bool)this.AlgGreedyRadio.IsChecked == true)
            {
                at = new AlgTypeGreedy();
            }
            else
            {
                at = new AlgTypeACO((bool)this.AlgElitismCheckBox.IsChecked, (bool)this.AlgFootSteppingCheckBox.IsChecked, (bool)this.AlgMinMaxCheckBox.IsChecked, (bool)this.AlgSystemCheckBox.IsChecked);
            }
            PheromoneType pt = new PheromoneType();
            AntColony.Instance.Start(at, pt, sc);
            this.UpdateResults();
        }

        private void StartSeriesButton_Click(object sender, RoutedEventArgs e)
        {
            AlgType at;
            if ((bool)this.AlgGreedyRadio.IsChecked == true)
            {
                at = new AlgTypeGreedy();
            }
            else
            {
                at = new AlgTypeACO((bool)this.AlgElitismCheckBox.IsChecked, (bool)this.AlgFootSteppingCheckBox.IsChecked, (bool)this.AlgMinMaxCheckBox.IsChecked, (bool)this.AlgSystemCheckBox.IsChecked);
            } 
            Series sw = new Series(at);
            sw.ShowDialog();
            this.UpdateResults();
            this.ReadParameters();
        }

        #endregion

        #region Problem buttons

        private void CleanProblemButton_Click(object sender, RoutedEventArgs e)
        {
            AntColony.Instance.ClearProblem();
            this.UpdateResults();
            this.ReadParameters();
        }

        private void GenerateProblemButton_Click(object sender, RoutedEventArgs e)
        {
            GenerateProblem gp = new GenerateProblem();
            gp.ShowDialog();
            this.UpdateResults();
            this.ReadParameters();
        }

        private void LoadProblemButton_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog o = new OpenFileDialog();
            o.ShowDialog();
            AntColony.Instance.ReadProblemFromFile(o.FileName);
            this.UpdateResults();
            this.ReadParameters();
        }

        private void SaveProblemButton_Click(object sender, RoutedEventArgs e)
        {
            SaveFileDialog sf = new SaveFileDialog();
            sf.DefaultExt = "txt";
            sf.AddExtension = true;
            sf.ShowDialog();
            if (sf.FileName != null)
            {
                AntColony.Instance.SaveProblem(sf.FileName);
            }
        }

        private void SaveResultButton_Click(object sender, RoutedEventArgs e)
        {
            SaveFileDialog sf = new SaveFileDialog();
            sf.DefaultExt = "txt";
            sf.AddExtension = true;
            sf.ShowDialog();
            if (sf.FileName != null)
            {
                SaveDataClass.Instance.ResetSaveString();
                if ((bool)this.SaveResultAsReadableRadio.IsChecked == true)
                {
                    SaveDataClass.Instance.SaveReadableResults(sf.FileName);
                }
                else
                {
                    SaveDataClass.Instance.SaveTableResults(sf.FileName);
                }
            }
        }

        #endregion

        #region Update

        public void ReadParameters()
        {
            this.AgentBox.Text = AntColony.Instance.AgentNumber.ToString();
            this.AntBox.Text = AntColony.Instance.AntNumber.ToString();
            this.AlphaBox.Text = AntColony.Instance.Alpha.ToString();
            this.BetaBox.Text = AntColony.Instance.Beta.ToString();
            this.RhoBox.Text = AntColony.Instance.Rho.ToString();
            this.PheromoneBox.Text = AntColony.Instance.PheromoneFaktor.ToString();
        }

        private void SetParameters()
        {
            try
            {
                AntColony.Instance.AgentNumber = int.Parse(this.AgentBox.Text);
                AntColony.Instance.AntNumber = int.Parse(this.AntBox.Text);
                AntColony.Instance.Alpha = double.Parse(this.AlphaBox.Text);
                AntColony.Instance.Beta = double.Parse(this.BetaBox.Text);
                AntColony.Instance.Rho = double.Parse(this.RhoBox.Text);
                AntColony.Instance.PheromoneFaktor = double.Parse(this.PheromoneBox.Text);
            }
            catch
            {
                MessageBox.Show("Error in reading parameter values!\ncheck for characters and/or floating points in integer values");
            }
        }

        private void UndoParametersButton_Click(object sender, RoutedEventArgs e)
        {
            this.ReadParameters();
        }

        private void UpdateParametersButton_Click(object sender, RoutedEventArgs e)
        {
            this.SetParameters();
            this.ReadParameters();
        }

        public void UpdateResults()
        {
            this.ResultCyclusBox.Text = AntColony.Instance.CyclusCounter.ToString();
            this.ResultTimeBox.Text = AntColony.Instance.LastOptimizationDuration.ToString();
            this.ResultCityNumberBox.Text = AntColony.Instance.CityList.Count.ToString();
            if (AntColony.Instance.BestRouteList.Count == 0)
            {
                this.ResultBestRouteBox.Text = "-";
                this.ResultFoundAtBox.Text = "-";
            }
            else
            {
                this.ResultBestRouteBox.Text = AntColony.Instance.BestRouteList[AntColony.Instance.BestRouteList.Count - 1].routeLength.ToString();
                this.ResultFoundAtBox.Text = AntColony.Instance.BestRouteList[AntColony.Instance.BestRouteList.Count - 1].CyclusesNeeded.ToString();
            }
            if (AntColony.Instance.AverageRouteLength.Count == 0)
            {
                this.ResultAverageRouteBox.Text = "-";
            }
            else
            {
                List<double> lr = AntColony.Instance.AverageRouteLength;
                double sum = 0;
                foreach (double d in lr)
                {
                    sum += d;
                }
                this.ResultAverageRouteBox.Text = (sum / lr.Count).ToString();
            }
            this.ProblemTypeBox.Text = AntColony.Instance.ProblemTyp.ToString();
            if (AntColony.Instance.ProblemTyp == ProblemType.VRP)
            {
                this.AgentBox.IsEnabled = true;
            }
            else
            {
                this.AgentBox.IsEnabled = false;
            }
            this.DrawResults();
        }

        private void AgentBox_LostKeyboardFocus(object sender, KeyboardFocusChangedEventArgs e)
        {
            this.SetParameters();
            this.ReadParameters();
        }

        #endregion

        #region Graphic
        
        private void DrawResults()
        {
            this.DrawMap();
            this.DrawGraph();
        }

        private void AdjustMapFactors()
        {
            List<City> cityList = AntColony.Instance.CityList;
            double max = 0;
            foreach (City c in cityList)
            {
                if (c.X >max)
                {
                    max = c.X;
                }
                if (c.Y >max)
                {
                    max = c.Y;
                }
            }
            if (cityList.Count > 10)
            {
                if ((max / this.mapXfactor < 60) || (max / this.mapXfactor > 597))
                {
                    this.mapXfactor = max / 580;
                    this.mapYfactor = max / 580;
                }
            }
            if (cityList.Count == 0)
            {
                this.mapXfactor = 1.0 / 600.0;
                this.mapYfactor = 1.0 / 600.0;
            }
        }

        private void DrawMap()
        {
            this.DrawProblemCanvas.Children.Clear();
            List<City> cityList = AntColony.Instance.CityList;
            this.AdjustMapFactors();
            if (cityList.Count < 1)
            {
                return;
            }
            //draw citys
            if ((bool)this.DisplayCitysCheckBox.IsChecked == true)
            {
                foreach (City c in cityList)
                {
                    Ellipse circle = new Ellipse();
                    circle.Width = 4;
                    circle.Height = 4;
                    circle.Stroke = new SolidColorBrush(Colors.Blue);
                    circle.Fill = new SolidColorBrush(Colors.Blue);
                    circle.StrokeThickness = 1;
                    circle.SetValue(Canvas.LeftProperty, c.X / this.mapXfactor - 2);
                    circle.SetValue(Canvas.TopProperty, c.Y / this.mapYfactor - 2);
                    if (AntColony.Instance.ProblemTyp == ProblemType.VRP)
                    {
                        if (c == cityList[0])
                        {
                            circle.Width = 8;
                            circle.Height = 8;
                            circle.SetValue(Canvas.LeftProperty, c.X / this.mapXfactor - 4);
                            circle.SetValue(Canvas.TopProperty, c.Y / this.mapYfactor - 4);
                            circle.Fill = new SolidColorBrush(Colors.Yellow);
                        }
                    }
                    this.DrawProblemCanvas.Children.Add(circle);
                }
            }
            //draw pheromone
            if (((bool)this.DisplayTrailCheckBox.IsChecked == true)&&(cityList.Count>1))
            {
                //max in trail
                double max = 0;
                for (int i = 0; i < cityList.Count; i++)
                {
                    for (int j = i + 1; j < cityList.Count; j++)
                    {
                        if (AntColony.Instance.trail[i, j] > max)
                        {
                            max = AntColony.Instance.trail[i, j];
                        }
                    }
                }
                for (int i = 0; i < cityList.Count; i++)
                {
                    for (int j = i + 1; j < cityList.Count; j++)
                    {
                        Line TrailLine = new Line();
                        TrailLine.Stroke = new SolidColorBrush(Colors.Black);
                        TrailLine.StrokeThickness = 1;
                        //TrailLine.Opacity = AntColony.Instance.trail[i, j] / max;
                        TrailLine.Opacity = (2 * AntColony.Instance.trail[i, j] / max) / (1 + AntColony.Instance.trail[i, j] / max);
                        TrailLine.X1 = cityList[i].X / this.mapXfactor;
                        TrailLine.Y1 = cityList[i].Y / this.mapYfactor;
                        TrailLine.X2 = cityList[j].X / this.mapXfactor;
                        TrailLine.Y2 = cityList[j].Y / this.mapYfactor;
                        this.DrawProblemCanvas.Children.Add(TrailLine);
                    }
                }
            }
            //draw best route
            if (((bool)this.DisplayBestRouteCheckBox.IsChecked == true)&&(AntColony.Instance.BestRouteCityRow.Count>0))
            {
                List<int> best = AntColony.Instance.BestRouteCityRow;
                switch (AntColony.Instance.ProblemTyp)
                {
                    case ProblemType.TSP:
                        #region TSP polyline
                        Point p = new Point();
                        Polyline line = new Polyline();
                        foreach (int index in best)
                        {
                                p.X = cityList[index].X / this.mapXfactor;
                                p.Y = cityList[index].Y / this.mapYfactor;
                                    line.Points.Add(p);
                        };
                        p.X = cityList[best[0]].X / this.mapXfactor;
                        p.Y = cityList[best[0]].Y / this.mapYfactor;
                        line.Points.Add(p);
                        line.Stroke = new SolidColorBrush(Colors.Red);
                        line.StrokeThickness = 2;
                        line.Opacity = 1;
                        this.DrawProblemCanvas.Children.Add(line);
                        break;
                        #endregion
                    case ProblemType.VRP:
                        #region VRP lines
                        Color[] colors = new Color[8];
                        colors[0] = Colors.Red;
                        colors[1] = Colors.Purple;
                        colors[2] = Colors.Black;
                        colors[3] = Colors.Brown;
                        colors[4] = Colors.Green;
                        colors[5] = Colors.Orange;
                        colors[6] = Colors.Gray;
                        colors[7] = Colors.Yellow;
                        Point start = new Point();
                        start.X = cityList[best[0]].X / this.mapXfactor;
                        start.Y = cityList[best[0]].Y / this.mapYfactor;
                        Polyline[] Lines = new Polyline[AntColony.Instance.AgentNumber+1];
                        for (int i = 0; i < AntColony.Instance.AgentNumber+1; i++)
                        {
                            Lines[i] = new Polyline();
                        }
                        int lineNumber = 0;
                        Point pt = new Point();
                        foreach (int index in best)
                        {
                            pt.X = cityList[index].X / this.mapXfactor;
                            pt.Y = cityList[index].Y / this.mapYfactor;
                            Lines[lineNumber].Points.Add(pt);
                            if ((pt == start) && (Lines[lineNumber].Points.Count > 1))
                            {
                                lineNumber++;
                                Lines[lineNumber].Points.Add(pt);
                            }
                        }
                        int col = 0;
                        for (int j = 0; j < lineNumber; j++)
                        {
                            col = j;
                            while (col > 7)
                            {
                                col -= 8;
                            }
                            Lines[j].Stroke = new SolidColorBrush(colors[col]);
                            Lines[j].StrokeThickness = 2;
                            Lines[j].Opacity = 1;
                            this.DrawProblemCanvas.Children.Add(Lines[j]);
                        }
                        break;
                        #endregion
                    default:
                        return;
                }
            }
        }

        private void DrawGraph()
        {
            this.DrawGraphCanvas.Children.Clear();
            List<ResultFound> rf = AntColony.Instance.BestRouteList;
            if (rf.Count ==0)
            {
                return;
            }
            double heightFactor = 100.0 / rf[0].routeLength;
            double widthFactor = 510.0 / AntColony.Instance.CyclusCounter;
            #region Numbers
            // max
            TextBlock max = new TextBlock();
            max.Text = ((float)rf[0].routeLength).ToString();
            max.FontSize = 10;
            max.Opacity = 1;
            max.Foreground = new SolidColorBrush(Colors.Black);
            max.SetValue(Canvas.LeftProperty, 2.0);
            max.SetValue(Canvas.TopProperty, 5.0);
            this.DrawGraphCanvas.Children.Add(max);
            // min
            TextBlock min = new TextBlock();
            min.Text = "0";
            min.FontSize = 10;
            min.Opacity = 1;
            min.Foreground = new SolidColorBrush(Colors.Black);
            min.SetValue(Canvas.LeftProperty, 21.0);
            min.SetValue(Canvas.TopProperty, 100.0);
            this.DrawGraphCanvas.Children.Add(min);
            // opt
            TextBlock opt = new TextBlock();
            opt.Text = ((float)rf[rf.Count-1].routeLength).ToString();
            opt.FontSize = 10;
            opt.Opacity = 1;
            opt.Foreground = new SolidColorBrush(Colors.Black);
            opt.SetValue(Canvas.LeftProperty, 1.0);
            opt.SetValue(Canvas.TopProperty, (double)(110 - rf[rf.Count - 1].routeLength * heightFactor));
            this.DrawGraphCanvas.Children.Add(opt);
            #endregion
            if ((bool)this.DisplayGraphOptimumCheckBox.IsChecked == true)
            {
                Line optima = new Line();
                optima.Stroke = new SolidColorBrush(Colors.Blue);
                optima.StrokeThickness = 2;
                optima.Opacity = 1;
                optima.X1 = 50;
                optima.X2 = 50 + AntColony.Instance.CyclusCounter * widthFactor;
                optima.Y1 = 117 - (rf[rf.Count - 1].routeLength * heightFactor);
                optima.Y2 = optima.Y1;
                this.DrawGraphCanvas.Children.Add(optima);
            }
            if ((bool)this.DisplayGraphBestCheckBox.IsChecked == true)
            {
                Polyline BestRoutes = new Polyline();
                Point p = new Point(0, 0);
                BestRoutes.Stroke = new SolidColorBrush(Colors.Red);
                BestRoutes.StrokeThickness = 2;
                BestRoutes.Opacity = 1;
                foreach (ResultFound r in rf)
                {
                    p.X = 50 + r.CyclusesNeeded * widthFactor;
                    if (BestRoutes.Points.Count > 0)
                    {
                        BestRoutes.Points.Add(p);
                    }
                    p.Y = 115 - (r.routeLength * heightFactor);
                    BestRoutes.Points.Add(p);
                }
                p.X = 50 + AntColony.Instance.CyclusCounter * widthFactor;
                BestRoutes.Points.Add(p);
                this.DrawGraphCanvas.Children.Add(BestRoutes);
            }
            if ((bool)this.DisplayGraphAverageCheckBox.IsChecked == true)
            {
                Polyline Avg = new Polyline();
                Point p = new Point(0, 0);
                Avg.Stroke = new SolidColorBrush(Colors.Green);
                Avg.StrokeThickness = 2;
                Avg.Opacity = 1;
                List<double> ar = AntColony.Instance.AverageRouteLength;
                int i = 0;
                foreach (double d in ar)
                {
                    p.X = 50 + i * widthFactor;
                    i++;
                    p.Y = 115 - d * heightFactor;
                    if (p.Y < 3)
                    {
                        p.Y = 3;
                    }
                    Avg.Points.Add(p);
                }
                this.DrawGraphCanvas.Children.Add(Avg);
            }
        }

        #endregion

        #region radio faders

        private void AlgAntRadio_Checked(object sender, RoutedEventArgs e)
        {
            this.AlphaBox.IsEnabled = true;
            this.BetaBox.IsEnabled = true;
            this.RhoBox.IsEnabled = true;
            this.PheromoneBox.IsEnabled = true;
        }

        private void AlgGreedyRadio_Checked(object sender, RoutedEventArgs e)
        {
            this.AlphaBox.IsEnabled = false;
            this.BetaBox.IsEnabled = false;
            this.RhoBox.IsEnabled = false;
            this.PheromoneBox.IsEnabled = false;
        }

        #endregion

    }
}
